import microbit

# (c) IDWizard 2017
# MIT License.

microbit.display.off()


LOW = 0
HIGH = 1
FULL_ROTATION = int(4075.7728395061727 / 8)

# servo positions
UP = 45
DOWN = 90

HALF_STEP = [
    [LOW, LOW, LOW, HIGH],
    [LOW, LOW, HIGH, HIGH],
    [LOW, LOW, HIGH, LOW],
    [LOW, HIGH, HIGH, LOW],
    [LOW, HIGH, LOW, LOW],
    [HIGH, HIGH, LOW, LOW],
    [HIGH, LOW, LOW, LOW],
    [HIGH, LOW, LOW, HIGH],
]

FULL_STEP = [
 [HIGH, LOW, HIGH, LOW],
 [LOW, HIGH, HIGH, LOW],
 [LOW, HIGH, LOW, HIGH],
 [HIGH, LOW, LOW, HIGH]
]

class Command():
    """Tell a stepper to move X many steps in direction"""
    def __init__(self, stepper, steps, direction=1):
        self.stepper = stepper
        self.steps = steps
        self.direction = direction

class Driver():
    """Drive a set of motors, each with their own commands"""

    @staticmethod
    def run(commands):
        """Takes a list of commands and interleaves their step calls"""

        # Work out total steps to take
        max_steps = sum([c.steps for c in commands])

        count = 0
        while count != max_steps:
            for command in commands:
                # we want to interleave the commands
                if command.steps > 0:
                    command.stepper.step(1, command.direction)
                    command.steps -= 1
                    count += 1


class Stepper():
    def __init__(self, mode, pin1, pin2, pin3, pin4, delay=2):
        self.mode = mode
        self.pin1 = pin1
        self.pin2 = pin2
        self.pin3 = pin3
        self.pin4 = pin4
        self.delay = delay

        # Initialize all to 0
        self.reset()

    def step(self, count, direction=1):
        """Rotate count steps. direction = -1 means backwards"""
        for x in range(count):
            for bit in self.mode[::direction]:
                self.pin1.write_digital(bit[0])
                self.pin2.write_digital(bit[1])
                self.pin3.write_digital(bit[2])
                self.pin4.write_digital(bit[3])
                microbit.sleep(self.delay)
        self.reset()

    def reset(self):
        # Reset to 0, no holding, these are geared, you can't move them
        self.pin1.write_digital(0)
        self.pin2.write_digital(0)
        self.pin3.write_digital(0)
        self.pin4.write_digital(0)

class Servo():
    def __init__(self, pin):
        self.pin = pin
        self.pin.set_analog_period(20)

    def write_pos(self, pos):
        self.pin.write_analog(pos)

class Mirobot():
    def __init__(self, servo_pin=microbit.pin0, \
    stepper_left_pin1=microbit.pin13, stepper_left_pin2=microbit.pin14, stepper_left_pin3=microbit.pin15, stepper_left_pin4=microbit.pin16, \
    stepper_right_pin1=microbit.pin6, stepper_right_pin2=microbit.pin7, stepper_right_pin3=microbit.pin8, stepper_right_pin4=microbit.pin9):
        self.servo = Servo(servo_pin)
        self.left_stepper = Stepper(HALF_STEP, stepper_left_pin1, stepper_left_pin2, stepper_left_pin3, stepper_left_pin4, delay=3)
        self.right_stepper = Stepper(HALF_STEP, stepper_right_pin1, stepper_right_pin2, stepper_right_pin3, stepper_right_pin4, delay=3)


    def pen_up(self):
        self.servo.write_pos(UP)

    def pen_down(self):
        self.servo.write_pos(DOWN)

    def sleep(self, time):
        microbit.sleep(time)

    def drive(self, steps, direction1, direction2):
        for i in range(steps):
            self.left_stepper.step(1, direction1)
            self.right_stepper.step(1, direction2)

    def forward(self, steps):
        self.drive(steps, 1, 1)
        #self.left_stepper.reset()
        #self.right_stepper.reset()

    def backward(self, steps):
        self.drive(steps, -1, -1)

    def left(self, steps):
        self.drive(steps, -1, 1)

    def right(self, steps):
        self.drive(steps, 1, -1)



#Skirv koden deres under denne linja!!!!
m = Mirobot()
m.pen_up()
m.sleep(1000)
m.pen_down()
m.forward(FULL_ROTATION/4)
m.sleep(1000)
m.backward(FULL_ROTATION/4)
m.sleep(1000)
m.left(FULL_ROTATION/4)
m.sleep(1000)
m.right(FULL_ROTATION/4)
m.sleep(1000)